package
{	
	import flash.display.BitmapData;
	import flash.display.Sprite;
	import flash.geom.Point;
	import flash.sampler.NewObjectSample;
	
	import starling.animation.Transitions;
	import starling.animation.Tween;
	import starling.core.Starling;
	import starling.display.DisplayObject;
	import starling.display.Image;
	import starling.display.MovieClip;
	import starling.display.Sprite;
	import starling.events.Event;
	import starling.events.Touch;
	import starling.events.TouchEvent;
	import starling.events.TouchPhase;
	import starling.extensions.PDParticleSystem;
	import starling.textures.Texture;
	import starling.textures.TextureAtlas;
	
	public class Game extends Sprite
	{
		//embed assets
		
		//Particle texture and atlas
		[Embed(source="assets/particle.pex", mimeType="application/octet-stream")] private var ParticleXml:Class;
		[Embed(source="assets/texture.png")] private var ParticleTexture:Class;
		
		//Spritesheet and atlas for blitting of animations
		[Embed(source="assets/spritesheet.xml", mimeType="application/octet-stream")] private var AtlasXml:Class;
		[Embed(source="assets/spritesheet.png")] private var AtlasTexture:Class;
		
		//robot static png
		[Embed(source="assets/robot-mio.png")] private var RobotGraphic:Class;
		
		private var atlas:TextureAtlas;
		
		public var particles:PDParticleSystem;
		public var robot:Image;
		
		public var enemies:Vector.<MovieClip>;
		public var starfield:Vector.<Image>;
		
		public var wave_ready:Boolean = true;
		public var tindex:uint
		public var enemies_wave:uint;
		public var enemyindex:uint;
		
		public var bomb:Boolean=false;
		
		private var _wave:uint = 0;
		private const NUM_ENEMIES:uint = 256;
		
		
		public var contador_wave:uint=0;
		
		public function Game()
		{
			super();
			
			addEventListener(Event.ADDED_TO_STAGE, addedToStage_handler);
		}
		
		
		private function addedToStage_handler(e:Event):void
		{
			
			trace("onAdded to STAGE");
			
			//sistema de partículas
			particles = new PDParticleSystem(XML(new ParticleXml), Texture.fromBitmap(new ParticleTexture()));
			particles.start();
			addChildAt(particles,0);
			
			//añadimos las partículas al juggler de Starling
			//para que estén animadas
			Starling.juggler.add(particles);
			
			//personaje robot
			robot= new Image(Texture.fromBitmap(new RobotGraphic));
			addChild(robot);
			
			robot.x = 100;
			robot.y = 100;
			
			addEventListener(Event.ENTER_FRAME, enterFrame_handler);
			stage.addEventListener(TouchEvent.TOUCH, touch_handler);
			
			//Blitting de animación de escudo html5 (enemigos)
			var texture:Texture = Texture.fromBitmap(new AtlasTexture());
			var xml:XML = XML(new AtlasXml());
			
			atlas = new TextureAtlas(texture,xml);
			
			enemies = new Vector.<MovieClip>();
			
			//generamos enemigos
			for(var i:uint=0;i <NUM_ENEMIES;i++)
			{
				enemies[i]= new MovieClip(atlas.getTextures("frame"),24);	
				
			}
			
			addBg();
			
		}
		
		//generamos el background de estrellas
		public function addBg():void
		{
			starfield = new Vector.<Image>();
			
			var star:flash.display.Sprite = new flash.display.Sprite();
			
			var star2:flash.display.Sprite = new flash.display.Sprite();
			star.graphics.beginFill(0xffffff,1);
			star2.graphics.beginFill(0xffffff,1);
			
			for(var i:uint=0;i<50;i++)
			{
				star.graphics.drawCircle(Math.random()*600,Math.random()*600,2*Math.random()+0.33);
				
				star2.graphics.drawCircle(Math.random()*600,Math.random()*600,2*Math.random()+0.33);	
				
			}
			
			star.graphics.endFill();
			star2.graphics.endFill();
			
			var stardata:BitmapData = new BitmapData(600,600, true,0);
			stardata.draw(star);
			
			var starTexture:Texture = Texture.fromBitmapData(stardata);
			
			var stardata2:BitmapData = new BitmapData(600,600, true,0);
			stardata2.draw(star2);
			
			var starTexture2:Texture = Texture.fromBitmapData(stardata2);
			
			starfield[0] = new Image(starTexture2);
			addChild(starfield[0]);
			
			starfield[1] = new Image(starTexture);
			starfield[1].y = -600;
			addChild(starfield[1]);
			
		}
		
		
		public function tweenComplete_handler(...args):void
		{
			contador_wave++;
			
			if(contador_wave == enemies_wave)
				wave_ready = true;
			
			enemies[args[0]].dispose();
		}
		
		//touch/mouse events
		private function touch_handler(e:TouchEvent):void
		{
			//almacenamos los eventos touch/mouse en un vector
			var touch:Vector.<Touch> = e.getTouches(stage);
			
			var pos:Point = touch[0].getLocation(stage);
			
			//el robot sigue al dedo (o ratón)
			robot.x = touch[0].globalX - robot.width*0.5;
			robot.y = touch[0].globalY - robot.height*0.5+150;
			
			//en caso de Multitouch, hacemos DOBLE BOMBA 
			//(Destruye toda la wave de una)
			//en el "on release"
			if((touch.length>1) && (touch[1].phase == TouchPhase.ENDED) )
			{
				bomb = true;
			}
		}
		
		//LOOP principal
		private function enterFrame_handler(e:Event):void
		{
			//partículas siempre sigue al robot 
			particles.emitterX= robot.x + robot.width*0.5;
			particles.emitterY = robot.y + robot.height+20;
			
			//generación de oleada de enemigos
			if(wave_ready)
			{
				
				wave_ready=false;
				contador_wave=0;
				
				//generamos como máximo NUM_ENEMIES (256) enemigos
				if(_wave <=8)
					enemies_wave = Math.pow(2,_wave);
				else
					enemies_wave = NUM_ENEMIES;
						
				trace("WAVE: "+_wave+" enemies_wave : "+enemies_wave);
				
				for(enemyindex=0; enemyindex < enemies_wave; enemyindex++)
				{
					
					enemies[enemyindex].x = Math.random()*600;
					enemies[enemyindex].y = -100;
					enemies[enemyindex].visible=true;
					
					//OJO: esto no es la manera más eficiente
					// -> object pooling!!
					addChild(enemies[enemyindex]);
					
					//añadimos animación al juggler
					Starling.juggler.add(enemies[enemyindex]);
				
					//los enemigos se mueven en dos fases, que son dos tweens
					//OJO: (no es la manera más eficiente)
					var t:Tween = new Tween(enemies[enemyindex], 2.0, Transitions.EASE_IN);
					t.moveTo(Math.random()*600,350);
					t.delay = 0+enemyindex/20;
					
					var t2:Tween = new Tween(enemies[enemyindex], 2.0, Transitions.EASE_OUT);
					t2.moveTo(Math.random()*600,600);
						
					t2.onComplete = tweenComplete_handler;
					t2.onCompleteArgs = [enemyindex];
					t2.delay = 1.5 + enemyindex/20;
					
					//los tweens TAMBIÉN deben añadirse al juggler
					Starling.juggler.add(t);
					Starling.juggler.add(t2);
				}
				
				_wave++;
				
				
			}
			
			//comprobamos si enemigo debe explotar
			for(enemyindex=0; enemyindex < enemies_wave; enemyindex++)
			{
				//colision o doble bomba
				if(bomb || enemies[enemyindex].visible && robot.bounds.intersects(enemies[enemyindex].bounds))
				{
					
					//borramos enemigo
					enemies[enemyindex].visible=false; 
					
					if(enemies[enemyindex].y > -100)
					{
						
						//explota!!
						var explosion:MovieClip = new MovieClip(atlas.getTextures("explosion"),12);
						
						explosion.loop=false;
						explosion.addEventListener(Event.COMPLETE, dispose_explosion);
						
						addChild(explosion);
						
						explosion.x = enemies[enemyindex].x;
						explosion.y = enemies[enemyindex].y+20;
						Starling.juggler.add(explosion);
						
						var t3:Tween = new Tween(explosion, 2, Transitions.LINEAR);
						t3.animate("y",explosion.y+100);
						t3.fadeTo(0.5);
						Starling.juggler.add(t3);
					}
					
					
				}
			}
			
			//desactivamos bomba
			bomb=false;
			
			//movemos bg
			starfield[0].y+=5;
			starfield[1].y+=5;
			
			if(starfield[0].y==600)
				starfield[0].y=-600;
			
			if(starfield[1].y==600)
				starfield[1].y=-600;
			
			
		}
		
		private function dispose_explosion(e:Event):void
		{
			MovieClip(e.target).dispose();
		}
		
	}//end class
}//end package